
// SOLUÇÃO Exercício – um exemplo de solução, elaborado em 25/04/205 pelo monitor Washington Y. Y. Wu e revisado pelo monitor Daniel I. Zagroba e pelo Prof. J. M. Simão.

#include <iostream>
#include <cstring>

using namespace std;

class MinhaString {
public:
    MinhaString(const char* s = ""){
        setString(s);
    }

    //Funcao construtora da copia criada ao usar operador- (necessario para manter a independecia e persistencia do objeto na memoria)
    MinhaString(const MinhaString& other)  {
        setString(other.pStr);
    }

    ~MinhaString() {
        if (pStr)
        {
            //cout << "Destruindo: " << pStr << endl;
            delete[] pStr;
            //system ("Pause");
        }
        else
        {
            cout << "Destruindo MinhaString nula" << endl;
        }

    }

    //MinhaString&
    void operator=(const MinhaString& other) {
        if (this != &other) {
            delete[] pStr;
            setString(other.pStr);
        }
        //return *this;
    }


    const char* getString() const { //metodo getter, por isso const
        return pStr;
    }


    void operator=(const char* s) {
        if(s!=pStr)
        {
            delete[]pStr;
            tamanho = strlen (s);
            setString( s );
        }
    }

    //Operadores de igualdade/Desigualdade
    bool operator==(const MinhaString& s) const {
        if ( 0 == strcmp ( pStr, s.getString() ) )  //poderia utilizar somente return strcmp(pStr, s.getString()) == 0;
        {
            return true;
        }
        else
        {
            return false;
        }
    }


    bool operator!=(const MinhaString& s) const { //poderia utilizar somente return !(*this == s);
        if ( 0 != strcmp ( pStr, s.getString() ) ) {
            return true;
        }
        else
        {
            return false;
        }
    }


    // Operador de concatenacao
    //Como nao queremos alterar originalmente a strings, criamos uma nova
    MinhaString operator+(const char* s) const {
        size_t tamanho = strlen(pStr) + strlen(s) + 1; //atualiza o tamanho
        char* temp = new char[tamanho]; //cria novo array com novo tamanho
        strcpy(temp, pStr); //copia 1a string para o array
        strcat(temp, s);  //copia 2a string para o array
        MinhaString NovaString(temp); //nova string
        delete[] temp; // deleta array
        return NovaString; //retorna nova string
    }

    // Operador de concatenacao com outra MinhaString
    MinhaString operator+(const MinhaString& s) const {
        return *this + s.getString(); //retorna a concatencao da 1a string com a 2a
    }

    // Operador de subtracao
    MinhaString operator-(const char* s) const {
        const char* pos = strstr(pStr, s); ; //funcao para procurar se substring esta presente
        if (!pos)
            return *this;  // nao achou

        size_t inicio = pos - pStr;  //inicio da substring
        size_t fim = strlen(s); // fim da substring
        size_t novoTam = strlen(pStr) - fim; // tamanho da string ao deletar substring

        char* nova = new char[novoTam + 1];  //cria array com o novo tamanho
        strncpy(nova, pStr, inicio); // copia string antes da substring
        nova[inicio] = '\0'; // o enter
        strcat(nova, pStr + inicio + fim);  //copia string depois da substring

        //cout << "Criando Resultado!" << endl;     system ("Pause");
        MinhaString resultado(nova);  // cria nova Minhastring (usando a construtora para copia)

        //cout << "Retornando Resultado!" << endl;     system ("Pause");
        return resultado;
    }

    // Operador de subtracao com outra MinhaString
    MinhaString operator-(const MinhaString& s) const {
        return *this - s.getString(); //retorna
    }

    // Operador de indice
    char operator[](size_t index) const {
        return pStr[index]; //retorna o char que esta naquela posicao
    }

    const size_t getTamanho() const
    {
        return tamanho;
    }

private:
    void setString(const char* s) { //seta a string e o tamanho
        tamanho = strlen(s);
        pStr = new char[tamanho + 1];
        strcpy(pStr, s);
    }

private:
    size_t tamanho; //size_t para que o tamanho nao seja negativo
    char* pStr;
};

// Operador de saída
ostream& operator<<(ostream& saida, const MinhaString& s) {
    saida << s.getString();
    return saida;
}

// Operador de entrada
istream& operator>>(istream& entrada, MinhaString& s) {
    char buffer[1024];
    entrada.getline(buffer, 1024);
    s = buffer;
    return entrada;
}


int main() {

    MinhaString S1("Minha primeira string soh minha.");
    MinhaString S2A, S2B;

    S2A = "Operador de atribuicao sobrecarregado é muito útil.";
    S2B = S2A;

    cout << S1 << endl;
    cout << S2A << endl;
    cout << S2B << endl << endl;

    (S2A == S2B) ? (cout << "S2A eh igual a S2B!") : (cout << "S2A NAO eh igual a S2B!");
    cout << endl << endl;

    (S1 == S2B) ? (cout << "S1 eh igual a S2B!") : (cout << "S1 NAO eh igual a S2B!");
    cout << endl << endl;
    system ("Pause");

    //Operator !=
    (S1 !=S2B) ? (cout << "S1 nao eh igual a S2B!") : (cout << "S1 eh igual a S2B!");
    cout << endl << endl;
    system ("Pause");

    //Operator +
    MinhaString S3 = S1 + " Concatenada!";
    cout <<"\n"<< S3 << endl;
    system ("Pause");

    //Operator -
    MinhaString S4 = S3 - S1;
    cout <<"\n"<< S4 << endl;
    system ("Pause");

    //Operator >>
    MinhaString S5;
    cout << "Digite um texto para S5: ";
    cin >> S5;
    cout << S5 << endl;
    system ("Pause");

    MinhaString SA ("Oi minha gente" ), SB ("Oi ");
    //MinhaString SC ("Oi gente");

    for (int i = 0; i < SA.getTamanho(); i++)
    {
        cout << SA[i];
    }
    cout << endl;
    system ("Pause");

    SA = SA - " minha";
    cout << SA << endl;
    system ("Pause");

    SA = SA - SB;
    cout << SA << endl;
    cout << SB << endl;
    // cout << SC << endl;
    system ("Pause");

    return 0;
}
